@@ -133,8 +133,9 @@ module Str_generate_of_sexp = struct
       let iinh = `I inh in
       atoms, structs, iinh :: ainhs, iinh :: sinhs
     | Rtag (_, true, [ _ ]) | Rtag (_, _, _ :: _ :: _) ->
-      Location.raise_errorf ~loc "split_row_field/&"
-    | Rtag (_, false, []) -> assert false
+      Location.raise_errorf ~loc "unsupported: polymorphic variant intersection type"
+    | Rtag (_, false, []) ->
+      Location.raise_errorf ~loc "unsupported: polymorphic variant empty type"
   ;;
 
   let type_constr_of_sexp ?(internal = false) id args =
@@ -148,49 +149,54 @@ module Str_generate_of_sexp = struct
     : Conversion.t
     =
     let loc = typ.ptyp_loc in
-    match typ with
-    | _ when Option.is_some (Attribute.get Attrs.opaque typ) ->
-      Conversion.of_reference_exn [%expr Sexplib0.Sexp_conv.opaque_of_sexp]
-    | [%type: [%t? _] sexp_opaque] | [%type: _] ->
-      Conversion.of_reference_exn [%expr Sexplib0.Sexp_conv.opaque_of_sexp]
-    | [%type: [%t? ty1] sexp_list] ->
-      let arg1 =
-        Conversion.to_expression ~loc (type_of_sexp ~error_source ~typevars ty1)
-      in
-      Conversion.of_reference_exn [%expr Sexplib0.Sexp_conv.list_of_sexp [%e arg1]]
-    | [%type: [%t? ty1] sexp_array] ->
-      let arg1 =
-        Conversion.to_expression ~loc (type_of_sexp ~error_source ~typevars ty1)
-      in
-      Conversion.of_reference_exn [%expr Sexplib0.Sexp_conv.array_of_sexp [%e arg1]]
-    | { ptyp_desc = Ptyp_tuple tp; _ } ->
-      Conversion.of_lambda (tuple_of_sexp ~error_source ~typevars (loc, tp))
-    | { ptyp_desc = Ptyp_var parm; _ } ->
-      (match Map.find typevars parm with
-       | Some fresh -> Conversion.of_reference_exn (Fresh_name.expression fresh)
-       | None ->
-         Location.raise_errorf ~loc "ppx_sexp_conv: unbound type variable '%s" parm)
-    | { ptyp_desc = Ptyp_constr (id, args); _ } ->
-      let args =
-        List.map args ~f:(fun arg ->
-          Conversion.to_expression ~loc (type_of_sexp ~error_source ~typevars arg))
-      in
-      Conversion.of_reference_exn (type_constr_of_sexp ~loc ~internal id args)
-    | { ptyp_desc = Ptyp_arrow (_, _, _); _ } ->
-      Conversion.of_reference_exn [%expr Sexplib0.Sexp_conv.fun_of_sexp]
-    | { ptyp_desc = Ptyp_variant (row_fields, Closed, _); _ } ->
-      variant_of_sexp ~error_source ~typevars ?full_type (loc, row_fields)
-    | { ptyp_desc = Ptyp_poly (parms, poly_tp); _ } ->
-      poly_of_sexp ~error_source ~typevars parms poly_tp
-    | { ptyp_desc = Ptyp_variant (_, Open, _); _ }
-    | { ptyp_desc = Ptyp_object (_, _); _ }
-    | { ptyp_desc = Ptyp_class (_, _); _ }
-    | { ptyp_desc = Ptyp_alias (_, _); _ }
-    | { ptyp_desc = Ptyp_package _; _ }
-    | { ptyp_desc = Ptyp_extension _; _ } ->
-      Location.raise_errorf ~loc "Type unsupported for ppx [of_sexp] conversion"
+    match Ppxlib_jane.Jane_syntax.Core_type.of_ast typ with
+    | Some (Jtyp_tuple alist, (_ : attributes)) ->
+      Conversion.of_reference_exn
+        (labeled_tuple_of_sexp ~error_source ~typevars ~loc alist)
+    | Some (Jtyp_layout _, _) | None ->
+      (match typ with
+       | _ when Option.is_some (Attribute.get Attrs.opaque typ) ->
+         Conversion.of_reference_exn [%expr Sexplib0.Sexp_conv.opaque_of_sexp]
+       | [%type: [%t? _] sexp_opaque] | [%type: _] ->
+         Conversion.of_reference_exn [%expr Sexplib0.Sexp_conv.opaque_of_sexp]
+       | [%type: [%t? ty1] sexp_list] ->
+         let arg1 =
+           Conversion.to_expression ~loc (type_of_sexp ~error_source ~typevars ty1)
+         in
+         Conversion.of_reference_exn [%expr Sexplib0.Sexp_conv.list_of_sexp [%e arg1]]
+       | [%type: [%t? ty1] sexp_array] ->
+         let arg1 =
+           Conversion.to_expression ~loc (type_of_sexp ~error_source ~typevars ty1)
+         in
+         Conversion.of_reference_exn [%expr Sexplib0.Sexp_conv.array_of_sexp [%e arg1]]
+       | { ptyp_desc = Ptyp_tuple tp; _ } ->
+         Conversion.of_lambda (tuple_of_sexp ~error_source ~typevars (loc, tp))
+       | { ptyp_desc = Ptyp_var parm; _ } ->
+         (match Map.find typevars parm with
+          | Some fresh -> Conversion.of_reference_exn (Fresh_name.expression fresh)
+          | None ->
+            Location.raise_errorf ~loc "ppx_sexp_conv: unbound type variable '%s" parm)
+       | { ptyp_desc = Ptyp_constr (id, args); _ } ->
+         let args =
+           List.map args ~f:(fun arg ->
+             Conversion.to_expression ~loc (type_of_sexp ~error_source ~typevars arg))
+         in
+         Conversion.of_reference_exn (type_constr_of_sexp ~loc ~internal id args)
+       | { ptyp_desc = Ptyp_arrow (_, _, _); _ } ->
+         Conversion.of_reference_exn [%expr Sexplib0.Sexp_conv.fun_of_sexp]
+       | { ptyp_desc = Ptyp_variant (row_fields, Closed, _); _ } ->
+         variant_of_sexp ~error_source ~typevars ?full_type (loc, row_fields)
+       | { ptyp_desc = Ptyp_poly (parms, poly_tp); _ } ->
+         poly_of_sexp ~error_source ~typevars parms poly_tp
+       | { ptyp_desc = Ptyp_variant (_, Open, _); _ }
+       | { ptyp_desc = Ptyp_object (_, _); _ }
+       | { ptyp_desc = Ptyp_class (_, _); _ }
+       | { ptyp_desc = Ptyp_alias (_, _); _ }
+       | { ptyp_desc = Ptyp_package _; _ }
+       | { ptyp_desc = Ptyp_extension _; _ } ->
+         Location.raise_errorf ~loc "Type unsupported for ppx [of_sexp] conversion")
 
-  (* Conversion of tuples *)
+  (* Conversion of (unlabeled) tuples *)
   and tuple_of_sexp ~error_source ~typevars (loc, tps) =
     let fps = List.map ~f:(type_of_sexp ~error_source ~typevars) tps in
     let ({ bindings; arguments; converted } : Conversion.Apply_all.t) =
@@ -202,22 +208,60 @@ module Str_generate_of_sexp = struct
       --> pexp_let ~loc Nonrecursive bindings (pexp_tuple ~loc converted)
     ; Fresh_name.pattern fresh_sexp
       --> [%expr
-        Sexplib0.Sexp_conv_error.tuple_of_size_n_expected
-          [%e error_source ()]
-          [%e eint ~loc n]
-          [%e Fresh_name.expression fresh_sexp]]
+            Sexplib0.Sexp_conv_error.tuple_of_size_n_expected
+              [%e error_source ()]
+              [%e eint ~loc n]
+              [%e Fresh_name.expression fresh_sexp]]
     ]
 
+  (* Conversion of labeled tuples *)
+  and labeled_tuple_of_sexp ~error_source ~typevars ~loc alist =
+    assert (Labeled_tuple.is_valid alist);
+    let fields_expr =
+      List.fold_right
+        alist
+        ~init:[%expr Empty]
+        ~f:(fun (label_option, core_type) rest_expr ->
+        let name_expr = estring ~loc (Labeled_tuple.atom_of_label label_option) in
+        let conv_expr =
+          type_of_sexp ~error_source ~typevars core_type
+          |> Conversion.to_expression ~loc:core_type.ptyp_loc
+        in
+        [%expr
+          Field { name = [%e name_expr]; conv = [%e conv_expr]; rest = [%e rest_expr] }])
+    in
+    let create_expr =
+      let pats, exprs =
+        List.map alist ~f:(fun (label_option, _) ->
+          let name = Fresh_name.create ~loc "field" in
+          Fresh_name.pattern name, (label_option, Fresh_name.expression name))
+        |> List.unzip
+      in
+      let pat =
+        List.fold_right pats ~init:(punit ~loc) ~f:(fun pat1 pat2 ->
+          ppat_tuple ~loc [ pat1; pat2 ])
+      in
+      let expr = Ppxlib_jane.Jane_syntax.Labeled_tuples.expr_of ~loc exprs in
+      [%expr fun [%p pat] -> [%e expr]]
+    in
+    pexp_apply
+      ~loc
+      [%expr Sexplib0.Sexp_conv_labeled_tuple.labeled_tuple_of_sexp]
+      [ Labelled "caller", error_source ()
+      ; Labelled "fields", fields_expr
+      ; Labelled "create", create_expr
+      ]
+
   (* Generate code for matching included variant types *)
   and handle_variant_inh
-        ~error_source
-        ~typevars
-        ~fresh_atom
-        ~fresh__sexp
-        full_type
-        ~match_last
-        other_matches
-        inh
+    ~error_source
+    ~typevars
+    ~fresh_atom
+    ~fresh__sexp
+    full_type
+    ~match_last
+    other_matches
+    inh
     =
     let loc = inh.ptyp_loc in
     let func_expr = type_of_sexp ~error_source ~typevars ~internal:true inh in
@@ -236,7 +280,7 @@ module Str_generate_of_sexp = struct
               ~loc
               [%expr
                 ([%e Conversion.to_expression ~loc app]
-                 :> [%t replace_variables_by_underscores full_type])]
+                  :> [%t replace_variables_by_underscores full_type])]
               match_exc
       ]
     in
@@ -244,14 +288,14 @@ module Str_generate_of_sexp = struct
 
   (* Generate code for matching atomic variants *)
   and mk_variant_match_atom
-        ~error_source
-        ~typevars
-        ~fresh_atom
-        ~fresh__sexp
-        loc
-        full_type
-        rev_atoms_inhs
-        rev_structs
+    ~error_source
+    ~typevars
+    ~fresh_atom
+    ~fresh__sexp
+    loc
+    full_type
+    rev_atoms_inhs
+    rev_structs
     =
     let coll (other_matches, match_last) = function
       | `A (loc, cnstr) ->
@@ -280,16 +324,16 @@ module Str_generate_of_sexp = struct
 
   (* Match arguments of constructors (variants or sum types) *)
   and mk_cnstr_args_match
-        ~error_source
-        ~typevars
-        ~loc
-        ~is_variant
-        ~fresh__sexp
-        ~fresh__tag
-        ~fresh_sexp_args
-        cnstr
-        tps
-        row
+    ~error_source
+    ~typevars
+    ~loc
+    ~is_variant
+    ~fresh__sexp
+    ~fresh__tag
+    ~fresh_sexp_args
+    cnstr
+    tps
+    row
     =
     let cnstr vars_expr =
       if is_variant
@@ -351,15 +395,15 @@ module Str_generate_of_sexp = struct
 
   (* Generate code for matching structured variants *)
   and mk_variant_match_struct
-        ~error_source
-        ~typevars
-        ~fresh_atom
-        ~fresh__sexp
-        ~fresh_sexp_args
-        loc
-        full_type
-        rev_structs_inhs
-        rev_atoms
+    ~error_source
+    ~typevars
+    ~fresh_atom
+    ~fresh__sexp
+    ~fresh_sexp_args
+    loc
+    full_type
+    rev_structs_inhs
+    rev_atoms
     =
     let has_structs_ref = ref false in
     let coll (other_matches, match_last) = function
@@ -448,9 +492,9 @@ module Str_generate_of_sexp = struct
     ; ppat_alias
         ~loc
         [%pat?
-               Sexplib0.Sexp.List
-               (Sexplib0.Sexp.Atom [%p Fresh_name.pattern fresh_atom]
-                :: [%p maybe_sexp_args_patt])]
+          Sexplib0.Sexp.List
+            (Sexplib0.Sexp.Atom [%p Fresh_name.pattern fresh_atom]
+            :: [%p maybe_sexp_args_patt])]
         (Fresh_name.to_string_loc fresh__sexp)
       --> match_struct
     ; ppat_alias
@@ -458,14 +502,14 @@ module Str_generate_of_sexp = struct
         [%pat? Sexplib0.Sexp.List (Sexplib0.Sexp.List _ :: _)]
         (Fresh_name.to_string_loc fresh_sexp)
       --> [%expr
-        Sexplib0.Sexp_conv_error.nested_list_invalid_poly_var
-          [%e error_source ()]
-          [%e Fresh_name.expression fresh_sexp]]
+            Sexplib0.Sexp_conv_error.nested_list_invalid_poly_var
+              [%e error_source ()]
+              [%e Fresh_name.expression fresh_sexp]]
     ; ppat_alias ~loc [%pat? Sexplib0.Sexp.List []] (Fresh_name.to_string_loc fresh_sexp)
       --> [%expr
-        Sexplib0.Sexp_conv_error.empty_list_invalid_poly_var
-          [%e error_source ()]
-          [%e Fresh_name.expression fresh_sexp]]
+            Sexplib0.Sexp_conv_error.empty_list_invalid_poly_var
+              [%e error_source ()]
+              [%e Fresh_name.expression fresh_sexp]]
     ]
 
   (* Generate matching code for variants *)
@@ -483,11 +527,11 @@ module Str_generate_of_sexp = struct
           let call =
             [%expr
               ([%e
-                Conversion.to_expression
-                  ~loc
-                  (type_of_sexp ~error_source ~typevars ~internal:true inh)]
+                 Conversion.to_expression
+                   ~loc
+                   (type_of_sexp ~error_source ~typevars ~internal:true inh)]
                  [%e Fresh_name.expression fresh_sexp]
-               :> [%t replace_variables_by_underscores full_type])]
+                :> [%t replace_variables_by_underscores full_type])]
           in
           match row_fields with
           | [] -> call
@@ -516,11 +560,11 @@ module Str_generate_of_sexp = struct
       Conversion.of_lambda
         [ Fresh_name.pattern fresh_sexp
           --> [%expr
-            try [%e pexp_match ~loc (Fresh_name.expression fresh_sexp) top_match] with
-            | Sexplib0.Sexp_conv_error.No_variant_match ->
-              Sexplib0.Sexp_conv_error.no_matching_variant_found
-                [%e error_source ()]
-                [%e Fresh_name.expression fresh_sexp]]
+                try [%e pexp_match ~loc (Fresh_name.expression fresh_sexp) top_match] with
+                | Sexplib0.Sexp_conv_error.No_variant_match ->
+                  Sexplib0.Sexp_conv_error.no_matching_variant_found
+                    [%e error_source ()]
+                    [%e Fresh_name.expression fresh_sexp]]
         ])
     else Conversion.of_lambda top_match
 
@@ -787,13 +831,13 @@ module Str_generate_of_sexp = struct
   ;;
 
   let args_for_record_of_sexp
-        td
-        fields
-        ~loc
-        ~error_source
-        ~typevars
-        ~constructor
-        ~allow_extra_fields
+    td
+    fields
+    ~loc
+    ~error_source
+    ~typevars
+    ~constructor
+    ~allow_extra_fields
     =
     let caller_expr = error_source () in
     let allow_extra_fields_expr = ebool ~loc allow_extra_fields in
@@ -867,8 +911,8 @@ module Str_generate_of_sexp = struct
         ppat_alias
           ~loc
           [%pat?
-                 Sexplib0.Sexp.List
-                 (Sexplib0.Sexp.Atom [%p string_pat] :: [%p Fresh_name.pattern fresh_sexps])]
+            Sexplib0.Sexp.List
+              (Sexplib0.Sexp.Atom [%p string_pat] :: [%p Fresh_name.pattern fresh_sexps])]
           (Fresh_name.to_string_loc fresh_sexp)
         --> (pexp_apply
                ~loc
@@ -896,14 +940,14 @@ module Str_generate_of_sexp = struct
         ppat_alias
           ~loc
           [%pat?
-                 Sexplib0.Sexp.List
-                 (Sexplib0.Sexp.Atom
-                    [%p
-                      ppat_alias
-                        ~loc
-                        [%pat? [%p lcstr] | [%p str]]
-                        (Fresh_name.to_string_loc fresh__tag)]
-                  :: [%p Fresh_name.pattern fresh_sexp_args])]
+            Sexplib0.Sexp.List
+              (Sexplib0.Sexp.Atom
+                 [%p
+                   ppat_alias
+                     ~loc
+                     [%pat? [%p lcstr] | [%p str]]
+                     (Fresh_name.to_string_loc fresh__tag)]
+              :: [%p Fresh_name.pattern fresh_sexp_args])]
           (Fresh_name.to_string_loc fresh__sexp)
         --> mk_cnstr_args_match
               ~error_source
@@ -932,9 +976,9 @@ module Str_generate_of_sexp = struct
           [%pat? Sexplib0.Sexp.List (Sexplib0.Sexp.Atom ([%p lcstr] | [%p str]) :: _)]
           (Fresh_name.to_string_loc fresh_sexp)
         --> [%expr
-          Sexplib0.Sexp_conv_error.stag_no_args
-            [%e error_source ()]
-            [%e Fresh_name.expression fresh_sexp]]
+              Sexplib0.Sexp_conv_error.stag_no_args
+                [%e error_source ()]
+                [%e Fresh_name.expression fresh_sexp]]
       | { pcd_name = cnstr; pcd_args = Pcstr_tuple (_ :: _) | Pcstr_record _; _ } ->
         let lcstr = pstring ~loc (String.uncapitalize cnstr.txt) in
         let str = pstring ~loc cnstr.txt in
@@ -943,9 +987,9 @@ module Str_generate_of_sexp = struct
           [%pat? Sexplib0.Sexp.Atom ([%p lcstr] | [%p str])]
           (Fresh_name.to_string_loc fresh_sexp)
         --> [%expr
-          Sexplib0.Sexp_conv_error.stag_takes_args
-            [%e error_source ()]
-            [%e Fresh_name.expression fresh_sexp]])
+              Sexplib0.Sexp_conv_error.stag_takes_args
+                [%e error_source ()]
+                [%e Fresh_name.expression fresh_sexp]])
   ;;
 
   (* Generate matching code for sum types *)
@@ -958,22 +1002,22 @@ module Str_generate_of_sexp = struct
           [%pat? Sexplib0.Sexp.List (Sexplib0.Sexp.List _ :: _)]
           (Fresh_name.to_string_loc fresh_sexp)
         --> [%expr
-          Sexplib0.Sexp_conv_error.nested_list_invalid_sum
-            [%e error_source ()]
-            [%e Fresh_name.expression fresh_sexp]]
+              Sexplib0.Sexp_conv_error.nested_list_invalid_sum
+                [%e error_source ()]
+                [%e Fresh_name.expression fresh_sexp]]
       ; ppat_alias
           ~loc
           [%pat? Sexplib0.Sexp.List []]
           (Fresh_name.to_string_loc fresh_sexp)
         --> [%expr
-          Sexplib0.Sexp_conv_error.empty_list_invalid_sum
-            [%e error_source ()]
-            [%e Fresh_name.expression fresh_sexp]]
+              Sexplib0.Sexp_conv_error.empty_list_invalid_sum
+                [%e error_source ()]
+                [%e Fresh_name.expression fresh_sexp]]
       ; Fresh_name.pattern fresh_sexp
         --> [%expr
-          Sexplib0.Sexp_conv_error.unexpected_stag
-            [%e error_source ()]
-            [%e Fresh_name.expression fresh_sexp]]
+              Sexplib0.Sexp_conv_error.unexpected_stag
+                [%e error_source ()]
+                [%e Fresh_name.expression fresh_sexp]]
       ]
       |> Lifted.return
     ]
@@ -1077,9 +1121,9 @@ module Str_generate_of_sexp = struct
           let no_variant_match_mc =
             [ [%pat? Sexplib0.Sexp_conv_error.No_variant_match]
               --> [%expr
-                Sexplib0.Sexp_conv_error.no_matching_variant_found
-                  [%e error_source ()]
-                  [%e Fresh_name.expression fresh_sexp]]
+                    Sexplib0.Sexp_conv_error.no_matching_variant_found
+                      [%e error_source ()]
+                      [%e Fresh_name.expression fresh_sexp]]
             ]
           in
           let internal_call =
