@@ -52,45 +52,49 @@ module Str_generate_sexp_of = struct
   (* Conversion of types *)
   let rec sexp_of_type ~renaming typ : Conversion.t =
     let loc = { typ.ptyp_loc with loc_ghost = true } in
-    match typ with
-    | _ when Option.is_some (Attribute.get Attrs.opaque typ) ->
-      Conversion.of_reference_exn [%expr Sexplib0.Sexp_conv.sexp_of_opaque]
-    | [%type: _] ->
-      Conversion.of_lambda [ ppat_any ~loc --> [%expr Sexplib0.Sexp.Atom "_"] ]
-    | [%type: [%t? _] sexp_opaque] ->
-      Conversion.of_reference_exn [%expr Sexplib0.Sexp_conv.sexp_of_opaque]
-    | { ptyp_desc = Ptyp_tuple tp; _ } ->
-      Conversion.of_lambda [ sexp_of_tuple ~renaming (loc, tp) ]
-    | { ptyp_desc = Ptyp_var parm; _ } ->
-      (match Renaming.binding_kind renaming parm ~loc with
-       | Universally_bound fresh ->
-         Conversion.of_reference_exn (Fresh_name.expression fresh)
-       | Existentially_bound -> sexp_of_type ~renaming [%type: _])
-    | { ptyp_desc = Ptyp_constr (id, args); _ } ->
-      Conversion.of_reference_exn
-        (sexp_of_type_constr
-           ~loc
-           id
-           (List.map args ~f:(fun tp ->
-              Conversion.to_expression ~loc (sexp_of_type ~renaming tp))))
-    | { ptyp_desc = Ptyp_arrow (_, _, _); _ } ->
-      Conversion.of_lambda
-        [ ppat_any ~loc
-          --> [%expr Sexplib0.Sexp_conv.sexp_of_fun Sexplib0.Sexp_conv.ignore]
-        ]
-    | { ptyp_desc = Ptyp_variant (row_fields, Closed, _); _ } ->
-      sexp_of_variant ~renaming (loc, row_fields)
-    | { ptyp_desc = Ptyp_poly (parms, poly_tp); _ } ->
-      sexp_of_poly ~renaming parms poly_tp
-    | { ptyp_desc = Ptyp_variant (_, Open, _); _ }
-    | { ptyp_desc = Ptyp_object (_, _); _ }
-    | { ptyp_desc = Ptyp_class (_, _); _ }
-    | { ptyp_desc = Ptyp_alias (_, _); _ }
-    | { ptyp_desc = Ptyp_package _; _ }
-    | { ptyp_desc = Ptyp_extension _; _ } ->
-      Location.raise_errorf ~loc "Type unsupported for ppx [sexp_of] conversion"
+    match Ppxlib_jane.Jane_syntax.Core_type.of_ast typ with
+    | Some (Jtyp_tuple alist, (_ : attributes)) ->
+      Conversion.of_lambda [ sexp_of_labeled_tuple ~renaming ~loc alist ]
+    | Some (Jtyp_layout _, _) | None ->
+      (match typ with
+       | _ when Option.is_some (Attribute.get Attrs.opaque typ) ->
+         Conversion.of_reference_exn [%expr Sexplib0.Sexp_conv.sexp_of_opaque]
+       | [%type: _] ->
+         Conversion.of_lambda [ ppat_any ~loc --> [%expr Sexplib0.Sexp.Atom "_"] ]
+       | [%type: [%t? _] sexp_opaque] ->
+         Conversion.of_reference_exn [%expr Sexplib0.Sexp_conv.sexp_of_opaque]
+       | { ptyp_desc = Ptyp_tuple tp; _ } ->
+         Conversion.of_lambda [ sexp_of_tuple ~renaming (loc, tp) ]
+       | { ptyp_desc = Ptyp_var parm; _ } ->
+         (match Renaming.binding_kind renaming parm ~loc with
+          | Universally_bound fresh ->
+            Conversion.of_reference_exn (Fresh_name.expression fresh)
+          | Existentially_bound -> sexp_of_type ~renaming [%type: _])
+       | { ptyp_desc = Ptyp_constr (id, args); _ } ->
+         Conversion.of_reference_exn
+           (sexp_of_type_constr
+              ~loc
+              id
+              (List.map args ~f:(fun tp ->
+                 Conversion.to_expression ~loc (sexp_of_type ~renaming tp))))
+       | { ptyp_desc = Ptyp_arrow (_, _, _); _ } ->
+         Conversion.of_lambda
+           [ ppat_any ~loc
+             --> [%expr Sexplib0.Sexp_conv.sexp_of_fun Sexplib0.Sexp_conv.ignore]
+           ]
+       | { ptyp_desc = Ptyp_variant (row_fields, Closed, _); _ } ->
+         sexp_of_variant ~renaming (loc, row_fields)
+       | { ptyp_desc = Ptyp_poly (parms, poly_tp); _ } ->
+         sexp_of_poly ~renaming parms poly_tp
+       | { ptyp_desc = Ptyp_variant (_, Open, _); _ }
+       | { ptyp_desc = Ptyp_object (_, _); _ }
+       | { ptyp_desc = Ptyp_class (_, _); _ }
+       | { ptyp_desc = Ptyp_alias (_, _); _ }
+       | { ptyp_desc = Ptyp_package _; _ }
+       | { ptyp_desc = Ptyp_extension _; _ } ->
+         Location.raise_errorf ~loc "Type unsupported for ppx [sexp_of] conversion")
 
-  (* Conversion of tuples *)
+  (* Conversion of (unlabeled) tuples *)
   and sexp_of_tuple ~renaming (loc, tps) =
     let fps = List.map ~f:(fun tp -> sexp_of_type ~renaming tp) tps in
     let ({ bindings; arguments; converted } : Conversion.Apply_all.t) =
@@ -100,6 +104,33 @@ module Str_generate_sexp_of = struct
     let expr = pexp_let ~loc Nonrecursive bindings in_expr in
     ppat_tuple ~loc arguments --> expr
 
+  (* Conversion of labeled tuples *)
+  and sexp_of_labeled_tuple ~renaming ~loc alist =
+    assert (Labeled_tuple.is_valid alist);
+    let ({ bindings; arguments; converted } : Conversion.Apply_all.t) =
+      List.map alist ~f:(fun (_, core_type) -> sexp_of_type ~renaming core_type)
+      |> Conversion.apply_all ~loc
+    in
+    let expr =
+      let sexp_exprs =
+        (* Constructor inference allows to to leave off [Sexplib0.Sexp.] here. *)
+        List.map2_exn alist converted ~f:(fun (label_option, _) expr ->
+          [%expr
+            List
+              [ Atom [%e estring ~loc (Labeled_tuple.atom_of_label label_option)]
+              ; [%e expr]
+              ]])
+      in
+      [%expr Sexplib0.Sexp.List [%e elist ~loc sexp_exprs]]
+      |> pexp_let ~loc Nonrecursive bindings
+    in
+    let pat =
+      ( List.map2_exn alist arguments ~f:(fun (label_option, _) arg -> label_option, arg)
+      , Closed )
+      |> Ppxlib_jane.Jane_syntax.Labeled_tuples.pat_of ~loc
+    in
+    pat --> expr
+
   (* Conversion of variant types *)
   and sexp_of_variant ~renaming ((loc, row_fields) : Location.t * row_field list)
     : Conversion.t
@@ -117,22 +148,22 @@ module Str_generate_sexp_of = struct
            let name = Fresh_name.create "l" ~loc in
            ppat_variant ~loc cnstr (Some (Fresh_name.pattern name))
            --> [%expr
-             Sexplib0.Sexp.List
-               (Sexplib0.Sexp.Atom [%e estring ~loc cnstr]
-                :: Sexplib0.Sexp_conv.list_map
-                     [%e cnv_expr]
-                     [%e Fresh_name.expression name])]
+                 Sexplib0.Sexp.List
+                   (Sexplib0.Sexp.Atom [%e estring ~loc cnstr]
+                    :: Sexplib0.Sexp_conv.list_map
+                         [%e cnv_expr]
+                         [%e Fresh_name.expression name])]
          | _ -> Attrs.invalid_attribute ~loc Attrs.list_poly "_ list")
       | Rtag ({ txt = cnstr; _ }, _, [ [%type: [%t? tp] sexp_list] ]) ->
         let cnv_expr = Conversion.to_expression ~loc (sexp_of_type ~renaming tp) in
         let name = Fresh_name.create "l" ~loc in
         ppat_variant ~loc cnstr (Some (Fresh_name.pattern name))
         --> [%expr
-          Sexplib0.Sexp.List
-            (Sexplib0.Sexp.Atom [%e estring ~loc cnstr]
-             :: Sexplib0.Sexp_conv.list_map
-                  [%e cnv_expr]
-                  [%e Fresh_name.expression name])]
+              Sexplib0.Sexp.List
+                (Sexplib0.Sexp.Atom [%e estring ~loc cnstr]
+                 :: Sexplib0.Sexp_conv.list_map
+                      [%e cnv_expr]
+                      [%e Fresh_name.expression name])]
       | Rtag ({ txt = cnstr; _ }, false, [ tp ]) ->
         let cnstr_expr = [%expr Sexplib0.Sexp.Atom [%e estring ~loc cnstr]] in
         let fresh = Fresh_name.create "v" ~loc in
@@ -146,16 +177,18 @@ module Str_generate_sexp_of = struct
         ppat_alias ~loc (ppat_type ~loc id) (Fresh_name.to_string_loc name)
         --> sexp_of_type_constr ~loc id [ Fresh_name.expression name ]
       | Rtag (_, true, [ _ ]) | Rtag (_, _, _ :: _ :: _) ->
-        Location.raise_errorf ~loc "unsupported: sexp_of_variant/Rtag/&"
+        Location.raise_errorf ~loc "unsupported: polymorphic variant intersection type"
       | Rinherit ({ ptyp_desc = Ptyp_constr (id, _ :: _); _ } as typ) ->
         let call = Conversion.to_expression ~loc (sexp_of_type ~renaming typ) in
         let name = Fresh_name.create "v" ~loc in
         ppat_alias ~loc (ppat_type ~loc id) (Fresh_name.to_string_loc name)
         --> [%expr [%e call] [%e Fresh_name.expression name]]
       | Rinherit _ ->
-        Location.raise_errorf ~loc "unsupported: sexp_of_variant/Rinherit/non-id"
-      (* impossible? *)
-      | Rtag (_, false, []) -> assert false
+        Location.raise_errorf
+          ~loc
+          "unsupported: polymorphic variant with invalid (non-identifier) inherited type"
+      | Rtag (_, false, []) ->
+        Location.raise_errorf ~loc "unsupported: polymorphic variant empty type"
     in
     Conversion.of_lambda (List.map ~f:item row_fields)
 
@@ -228,7 +261,7 @@ module Str_generate_sexp_of = struct
                       ]
                   in
                   ([%e Fresh_name.expression bnd] :: [%e Fresh_name.expression bnds]
-                   : _ Stdlib.List.t))]
+                    : _ Stdlib.List.t))]
             | Inspect_sexp is_empty_expr ->
               [%expr
                 let [%p Fresh_name.pattern arg] =
@@ -244,7 +277,7 @@ module Str_generate_sexp_of = struct
                       ]
                   in
                   ([%e Fresh_name.expression bnd] :: [%e Fresh_name.expression bnds]
-                   : _ Stdlib.List.t))]]
+                    : _ Stdlib.List.t))]]
         in
         [%e expr]]
     in
@@ -252,10 +285,10 @@ module Str_generate_sexp_of = struct
   ;;
 
   let disallow_type_variables_and_recursive_occurrences
-        ~types_being_defined
-        ~loc
-        ~attr_name
-        tp
+    ~types_being_defined
+    ~loc
+    ~attr_name
+    tp
     =
     let disallow_variables =
       let iter =
@@ -310,16 +343,16 @@ module Str_generate_sexp_of = struct
   ;;
 
   let sexp_of_default_field
-        ~types_being_defined
-        how
-        ~renaming
-        ~bnds
-        patt
-        expr
-        name
-        tp
-        ?sexp_of
-        default
+    ~types_being_defined
+    how
+    ~renaming
+    ~bnds
+    patt
+    expr
+    name
+    tp
+    ?sexp_of
+    default
     =
     let is_empty =
       let inspect_value equality_f =
@@ -329,23 +362,23 @@ module Str_generate_sexp_of = struct
       | Sexp ->
         Inspect_sexp
           (fun ~cnv_expr loc sexp_expr ->
-             [%expr Sexplib0.Sexp_conv.( = ) ([%e cnv_expr] [%e default]) [%e sexp_expr]])
+            [%expr Sexplib0.Sexp_conv.( = ) ([%e cnv_expr] [%e default]) [%e sexp_expr]])
         |> Lifted.return
       | No_arg ->
         inspect_value (fun loc ->
           [%expr
             Sexplib0.Sexp_conv.( = ) [@ocaml.ppwarning
-              "[@sexp_drop_default] is deprecated: please use \
-               one of:\n\
-               - [@sexp_drop_default f] and give an explicit \
-               equality function ([f = Poly.(=)] corresponds to \
-               the old behavior)\n\
-               - [@sexp_drop_default.compare] if the type \
-               supports [%compare]\n\
-               - [@sexp_drop_default.equal] if the type \
-               supports [%equal]\n\
-               - [@sexp_drop_default.sexp] if you want to \
-               compare the sexp representations\n"]])
+                                       "[@sexp_drop_default] is deprecated: please use \
+                                        one of:\n\
+                                        - [@sexp_drop_default f] and give an explicit \
+                                        equality function ([f = Poly.(=)] corresponds to \
+                                        the old behavior)\n\
+                                        - [@sexp_drop_default.compare] if the type \
+                                        supports [%compare]\n\
+                                        - [@sexp_drop_default.equal] if the type \
+                                        supports [%equal]\n\
+                                        - [@sexp_drop_default.sexp] if you want to \
+                                        compare the sexp representations\n"]])
         |> Lifted.return
       | Func lifted -> lifted >>| fun f -> inspect_value (fun _ -> f)
       | Compare ->
@@ -375,18 +408,18 @@ module Str_generate_sexp_of = struct
     let list_empty_expr =
       Inspect_value
         (fun loc lst ->
-           [%expr
-             match [%e lst] with
-             | [] -> true
-             | _ -> false])
+          [%expr
+            match [%e lst] with
+            | [] -> true
+            | _ -> false])
     in
     let array_empty_expr =
       Inspect_value
         (fun loc arr ->
-           [%expr
-             match [%e arr] with
-             | [||] -> true
-             | _ -> false])
+          [%expr
+            match [%e arr] with
+            | [||] -> true
+            | _ -> false])
     in
     let coll lifted ld =
       lifted
@@ -416,7 +449,7 @@ module Str_generate_sexp_of = struct
                     ]
                 in
                 ([%e Fresh_name.expression bnd] :: [%e Fresh_name.expression bnds]
-                 : _ Stdlib.List.t)
+                  : _ Stdlib.List.t)
             in
             [%e expr]]
         in
@@ -433,7 +466,7 @@ module Str_generate_sexp_of = struct
                   Sexplib0.Sexp.List [ Sexplib0.Sexp.Atom [%e estring ~loc name] ]
                 in
                 ([%e Fresh_name.expression bnd] :: [%e Fresh_name.expression bnds]
-                 : _ Stdlib.List.t))
+                  : _ Stdlib.List.t))
               else [%e Fresh_name.expression bnds]
             in
             [%e expr]]
@@ -508,12 +541,12 @@ module Str_generate_sexp_of = struct
                  ; [%e Fresh_name.expression arg]
                  ]
                :: [%e Fresh_name.expression bnds]
-               : _ Stdlib.List.t)]
+                : _ Stdlib.List.t)]
         in
         ( patt
         , [%expr
-          let [%p Fresh_name.pattern bnds] = [%e bnds_expr] in
-          [%e expr]] )
+            let [%p Fresh_name.pattern bnds] = [%e bnds_expr] in
+            [%e expr]] )
         |> Lifted.return
       | Specific Keep ->
         let tp = ld.pld_type in
@@ -529,12 +562,12 @@ module Str_generate_sexp_of = struct
                ; [%e Fresh_name.expression arg]
                ]
              :: [%e Fresh_name.expression bnds]
-             : _ Stdlib.List.t)]
+              : _ Stdlib.List.t)]
         in
         ( patt
         , [%expr
-          let [%p Fresh_name.pattern bnds] = [%e bnds_expr] in
-          [%e expr]] )
+            let [%p Fresh_name.pattern bnds] = [%e bnds_expr] in
+            [%e expr]] )
         |> Lifted.return
     in
     let init_expr = wrap_expr (Fresh_name.expression bnds) in
@@ -542,21 +575,21 @@ module Str_generate_sexp_of = struct
     >>| fun (patt, expr) ->
     ( ppat_record ~loc patt Closed
     , [%expr
-      let [%p Fresh_name.pattern bnds] = ([] : _ Stdlib.List.t) in
-      [%e expr]] )
+        let [%p Fresh_name.pattern bnds] = ([] : _ Stdlib.List.t) in
+        [%e expr]] )
   ;;
 
   (* Conversion of sum types *)
 
   let branch_sum
-        row
-        inline_attr
-        ~types_being_defined
-        renaming
-        ~loc
-        constr_lid
-        constr_str
-        args
+    row
+    inline_attr
+    ~types_being_defined
+    renaming
+    ~loc
+    constr_lid
+    constr_str
+    args
     =
     match args with
     | Pcstr_record lds ->
@@ -583,22 +616,22 @@ module Str_generate_sexp_of = struct
                let name = Fresh_name.create "l" ~loc in
                ppat_construct ~loc constr_lid (Some (Fresh_name.pattern name))
                --> [%expr
-                 Sexplib0.Sexp.List
-                   (Sexplib0.Sexp.Atom [%e constr_str]
-                    :: Sexplib0.Sexp_conv.list_map
-                         [%e cnv_expr]
-                         [%e Fresh_name.expression name])]
+                     Sexplib0.Sexp.List
+                       (Sexplib0.Sexp.Atom [%e constr_str]
+                        :: Sexplib0.Sexp_conv.list_map
+                             [%e cnv_expr]
+                             [%e Fresh_name.expression name])]
              | _ -> Attrs.invalid_attribute ~loc inline_attr "_ list")
           | [ [%type: [%t? tp] sexp_list] ] ->
             let cnv_expr = Conversion.to_expression ~loc (sexp_of_type ~renaming tp) in
             let name = Fresh_name.create "l" ~loc in
             ppat_construct ~loc constr_lid (Some (Fresh_name.pattern name))
             --> [%expr
-              Sexplib0.Sexp.List
-                (Sexplib0.Sexp.Atom [%e constr_str]
-                 :: Sexplib0.Sexp_conv.list_map
-                      [%e cnv_expr]
-                      [%e Fresh_name.expression name])]
+                  Sexplib0.Sexp.List
+                    (Sexplib0.Sexp.Atom [%e constr_str]
+                     :: Sexplib0.Sexp_conv.list_map
+                          [%e cnv_expr]
+                          [%e Fresh_name.expression name])]
           | _ ->
             let sexp_of_args = List.map ~f:(sexp_of_type ~renaming) args in
             let cnstr_expr = [%expr Sexplib0.Sexp.Atom [%e constr_str]] in
