@@ -274,48 +274,77 @@ let rec grammar_of_type core_type ~rec_flag ~tags_of_doc_comments =
     match from_attribute with
     | Some expr -> expr
     | None ->
-      (match core_type.ptyp_desc with
-       | Ptyp_any -> any_grammar ~loc "_"
-       | Ptyp_var name ->
-         (match rec_flag with
-          | Recursive ->
-            (* For recursive grammars, [grammar_of_type] for any type variables is called
-               inside a [defn]. The variables should therefore be resolved as [Tyvar]
-               grammars. *)
-            tyvar_grammar ~loc (estring ~loc name)
-          | Nonrecursive ->
-            (* Outside recursive [defn]s, type variables are passed in as function
-               arguments. *)
-            unapplied_type_constr_conv
-              ~loc
-              ~f:tyvar_grammar_name
-              (Located.lident ~loc name)
-            |> untyped_grammar ~loc)
-       | Ptyp_arrow _ -> arrow_grammar ~loc
-       | Ptyp_tuple list ->
-         List.map ~f:(grammar_of_type ~rec_flag ~tags_of_doc_comments) list
-         |> tuple_grammar ~loc
-         |> list_grammar ~loc
-       | Ptyp_constr (id, args) ->
-         List.map args ~f:(fun core_type ->
-           let loc = core_type.ptyp_loc in
-           grammar_of_type ~rec_flag ~tags_of_doc_comments core_type |> typed_grammar ~loc)
-         |> type_constr_conv ~loc ~f:grammar_name id
-         |> untyped_grammar ~loc
-       | Ptyp_object _ -> unsupported ~loc "object types"
-       | Ptyp_class _ -> unsupported ~loc "class types"
-       | Ptyp_alias _ -> unsupported ~loc "type aliases"
-       | Ptyp_variant (rows, closed_flag, (_ : string list option)) ->
-         (match closed_flag with
-          | Open -> unsupported ~loc "open polymorphic variant types"
-          | Closed ->
-            grammar_of_polymorphic_variant ~loc ~rec_flag ~tags_of_doc_comments rows)
-       | Ptyp_poly _ -> unsupported ~loc "explicitly polymorphic types"
-       | Ptyp_package _ -> unsupported ~loc "first-class module types"
-       | Ptyp_extension _ -> unsupported ~loc "unexpanded ppx extensions")
+      (match Ppxlib_jane.Jane_syntax.Core_type.of_ast core_type with
+       | Some (Jtyp_tuple ltps, _attrs) ->
+         grammar_of_labeled_tuple ~loc ~rec_flag ~tags_of_doc_comments ltps
+       | Some (Jtyp_layout _, _) | None ->
+         (match core_type.ptyp_desc with
+          | Ptyp_any -> any_grammar ~loc "_"
+          | Ptyp_var name ->
+            (match rec_flag with
+             | Recursive ->
+               (* For recursive grammars, [grammar_of_type] for any type variables is called
+                  inside a [defn]. The variables should therefore be resolved as [Tyvar]
+                  grammars. *)
+               tyvar_grammar ~loc (estring ~loc name)
+             | Nonrecursive ->
+               (* Outside recursive [defn]s, type variables are passed in as function
+                  arguments. *)
+               unapplied_type_constr_conv
+                 ~loc
+                 ~f:tyvar_grammar_name
+                 (Located.lident ~loc name)
+               |> untyped_grammar ~loc)
+          | Ptyp_arrow _ -> arrow_grammar ~loc
+          | Ptyp_tuple list ->
+            List.map ~f:(grammar_of_type ~rec_flag ~tags_of_doc_comments) list
+            |> tuple_grammar ~loc
+            |> list_grammar ~loc
+          | Ptyp_constr (id, args) ->
+            List.map args ~f:(fun core_type ->
+              let loc = core_type.ptyp_loc in
+              grammar_of_type ~rec_flag ~tags_of_doc_comments core_type
+              |> typed_grammar ~loc)
+            |> type_constr_conv ~loc ~f:grammar_name id
+            |> untyped_grammar ~loc
+          | Ptyp_object _ -> unsupported ~loc "object types"
+          | Ptyp_class _ -> unsupported ~loc "class types"
+          | Ptyp_alias _ -> unsupported ~loc "type aliases"
+          | Ptyp_variant (rows, closed_flag, (_ : string list option)) ->
+            (match closed_flag with
+             | Open -> unsupported ~loc "open polymorphic variant types"
+             | Closed ->
+               grammar_of_polymorphic_variant ~loc ~rec_flag ~tags_of_doc_comments rows)
+          | Ptyp_poly _ -> unsupported ~loc "explicitly polymorphic types"
+          | Ptyp_package _ -> unsupported ~loc "first-class module types"
+          | Ptyp_extension _ -> unsupported ~loc "unexpanded ppx extensions"))
   in
   grammar_of_type_tags core_type grammar ~tags_of_doc_comments
 
+and grammar_of_labeled_tuple ~loc ~rec_flag ~tags_of_doc_comments alist =
+  assert (Labeled_tuple.is_valid alist);
+  let fields =
+    List.concat_map alist ~f:(fun (lbl, typ) ->
+      let lbl = Labeled_tuple.atom_of_label lbl in
+      let field = grammar_of_type ~rec_flag ~tags_of_doc_comments typ in
+      let clauses : Variant_clause_type.t list =
+        (* Labeled tuples are encoded as a list of singleton variants, where the
+           constructor name is used for the label. *)
+        [ { name = { txt = lbl; loc }
+          ; comments = []
+          ; tags =
+              { defined_using_tags = None; defined_using_tag = [] }
+              (* We can use empty comments and tags because it's not possible to attach an
+             attribute to a labeled tuple field. *)
+          ; clause_kind = list_clause ~loc [%expr Cons ([%e field], Empty)]
+          }
+        ]
+      in
+      let case_sensitivity = [%expr Case_sensitive] in
+      variant_grammars ~loc ~case_sensitivity ~clauses)
+  in
+  list_grammar ~loc (tuple_grammar ~loc fields)
+
 and grammar_of_polymorphic_variant ~loc ~rec_flag ~tags_of_doc_comments rows =
   let inherits, clauses =
     List.partition_map rows ~f:(fun row : (_, Variant_clause_type.t) Either.t ->
@@ -509,8 +538,8 @@ let grammar_needs_lazy_wrapper expr =
 
 let lazy_grammar ~loc td expr =
   if List.is_empty td.ptype_params
-  (* polymorphic types generate functions, so the body does not need a [lazy] wrapper *)
-  && grammar_needs_lazy_wrapper expr
+     (* polymorphic types generate functions, so the body does not need a [lazy] wrapper *)
+     && grammar_needs_lazy_wrapper expr
   then [%expr Lazy (lazy [%e expr])]
   else expr
 ;;
